#!/bin/bash
# This script is written by GIJOE <gij@peak.ne.jp>
# New BSD License


#
# initials
#

read POST_RAW

declare -r my_default_name='hd_installer.cgi'
declare -r download_base_url='http://downloads.sourceforge.net/hodajuku/'
declare -r archive_default_name='hd_full_1_0_2.tar.gz'
declare -r trust_dirname='xoops_trust_path'
declare -r -a allowed_requests=('filename' 'trustindex' 'chmodexecphp' 'extract' 'overwrite')
declare -r -a trust_options=( `cd ..;pwd`"/$trust_dirname" `cd ../..;pwd`"/$trust_dirname" `cd ../../..;pwd`"/$trust_dirname" `pwd`"/$trust_dirname")
declare -r rootname_of_archive='html'
declare -r trustname_of_archive='xoops_trust_path'
declare -r command_after_extraction='chmod -R g-w *; chmod 0777 html/uploads/wizmobile 2>/dev/null'
declare -r tmp4install=`date +tmp_%Y%m%d`


REQUEST_rootpath=`pwd`
REQUEST_filename=''
REQUEST_trustindex=0
REQUEST_chmodexecphp=0
REQUEST_extract=''
REQUEST_overwrite=''


#
# functions
#

function check_environment() {
	# suExec enabled?
	if [ -O . ] ; then
		return 0;
	else
		return 1;
	fi
}

function fetch_request() {
	local tmp k_v key value allowed_key

	QUERY_STRING=`echo "$QUERY_STRING"|sed 's/[^a-zA-Z0-9.&=%_-]//g'`
	POST_RAW=`echo "$POST_RAW"|sed 's/[^a-zA-Z0-9.&=%_-]//g'`

	tmp="$QUERY_STRING&$POST_RAW&"
	until [ -z "$tmp" ] ; do
		k_v=${tmp%%&*}
		tmp=${tmp#*&}
		key=${k_v%%=*}
		value=${k_v#*=}
		for allowed_key in ${allowed_requests[@]}; do
			if [ "$allowed_key" = "$key" ] ; then
				eval "REQUEST_$allowed_key=$value"
			fi
		done
	done
}

function validate_requests() {
	# you must eliminate HTML special characters '"&<>
	REQUEST_filename=`echo "$REQUEST_filename"|sed 's/[^a-zA-Z0-9._-]//g'`
	# you can use let as intval() of php
	let REQUEST_trustindex=$REQUEST_trustindex
}

function html_header() {
	echo -e 'Content-Type: text/html'
	echo -e ''
	echo -e '<html>\n'
	echo -e '<head>\n'
	echo -e '<title>installer</title>\n'
	echo -e '</head>\n'
	echo -e '<body>\n'
}

function html_footer() {
	echo -e '</body>\n</html>'
}

function display_form() {
	echo '<form action="" method="post">'
	echo '<label>archive file name:'
	echo '<input type="text" name="filename" value="'$archive_default_name'" size="64" /></label><br />'
	echo 'tmp path: '$tmp4install'<br />'
	echo 'root path: '$REQUEST_rootpath'<br />'
	echo '<label>trust path: '
	echo '<select name="trustindex">'
	count=0
	for path in ${trust_options[@]}; do
		echo "<option value='$count'> $path </option>"
		let count=count+1
	done
	echo '</select></label><br />'
	echo 'need execute permissions for *.php?'
	echo '<label><input type="radio" name="chmodexecphp" value="0" checked="checked" />no</label>'
	echo '<label><input type="radio" name="chmodexecphp" value="1" />yes</label>'
	echo '<br />'
	echo '<input type="submit" value="submit" />'
	echo '</form>'
}

function display_confirm() {
	echo '<form action="" method="post">'

	if [ -r "$REQUEST_filename" ] ; then
		echo "archive(local): $REQUEST_filename<br />"
	else
		echo "archive(download): $download_base_url$REQUEST_filename<br />"
	fi

	echo 'tmp path: '$tmp4install'<br />'
	echo 'root path: '$REQUEST_rootpath'<br />'
	echo 'trust path: '${trust_options[$REQUEST_trustindex]}'<br />'
	echo '<input type="hidden" name="filename" value="'$REQUEST_filename'" />'
	echo '<input type="hidden" name="trustindex" value="'$REQUEST_trustindex'" />'
	echo '<input type="hidden" name="chmodexecphp" value="'$REQUEST_chmodexecphp'" />'
	echo '<input type="submit" name="extract" value="extract it" />'
	echo '</form>'
}

function extraction() {
	local from_remote

	# create tmp directory for this installation
	mkdir $tmp4install 2>/dev/null
	cd $tmp4install

	if [ -r "../$REQUEST_filename" ] ; then
		# 1st: local file
		cp -p "../$REQUEST_filename" $REQUEST_filename
		from_remote=false
	else
		# 2nd: try wget from URL
		wget -q $download_base_url$REQUEST_filename
		from_remote=true
	fi
	# 3rd: try curl from URL
	if [ ! -r $REQUEST_filename ] ; then
		curl -s -L $download_base_url$REQUEST_filename > $REQUEST_filename
	fi
	# last: raise an error
	if [ ! -r $REQUEST_filename ] ; then
		rm $REQUEST_filename 2>/dev/null
		echo "<p>cannot download $download_base_url$REQUEST_filename</p>"
		echo "<p>confirm the url</p>"
		exit 1
	fi

	# extract the archive
	if ! tar xpzf $REQUEST_filename ; then
		echo "<p>invalid archive type</p>"
		exit 1
	fi

	# check rootname
	if [ ! -x $rootname_of_archive ] ; then
		echo "<p>missing $rootname_of_archive folder in the archive </p>"
		exit 1
	fi

	# check trustname
	# if [ ! -x $trustname_of_archive ] ; then
	#	echo "<p>missing $trustname_of_archive folder in the archive </p>"
	#	exit 1
	# fi

	# chmod a+x *.php if requested
	if [ $REQUEST_chmodexecphp = '1' ] ; then
		find $rootname_of_archive -name '*.php' -exec chmod a+x {} \;
	fi

	# chmod some files/directories if necessary
	eval "$command_after_extraction"

	cd -

	# message
	if [ $from_remote = true ] ; then
		echo "<p>The archive has been fetched successfully</p>"
	fi
	echo "<p>Extraction is finished successfully</p>"

	# last form
	echo '<form action="" method="post">'

	echo 'tmp path: '$tmp4install'<br />'
	echo 'root path: '$REQUEST_rootpath'<br />'
	echo 'trust path: '${trust_options[$REQUEST_trustindex]}'<br />'
	echo '<input type="hidden" name="trustindex" value="'$REQUEST_trustindex'" />'
	echo '<input type="submit" name="overwrite" value="copy/overwrite" onclick="return confirm(&#039;Are you OK?&#039;);" />'
	echo '</form>'
}

function overwriting() {
	# mkdir XOOPS_TRUST_PATH
	mkdir ${trust_options[$REQUEST_trustindex]} 2>/dev/null
	cd $tmp4install

	# copy html/... to ROOT_PATH
	for i in $rootname_of_archive/* ; do # */
		cp -pR $i $REQUEST_rootpath
	done
	rm -r $rootname_of_archive

	# copy xoops_trust_path/... to TRUST_PATH if xoops_trust_path exsits
	if [ -x $trustname_of_archive ] ; then
		for i in $trustname_of_archive/* ; do # */
			cp -pR $i ${trust_options[$REQUEST_trustindex]}
		done
		cp -pR "$trustname_of_archive/.htaccess" ${trust_options[$REQUEST_trustindex]} 2>/dev/null
		rm -r $trustname_of_archive
	fi

	cd -

	# remove tmp directory for this installation
	rm -r $tmp4install

	# message
	echo "<p>All files are copied successfully</p>"
	echo "<p>Don&#039;t forget removing this CGI file</p>"
	echo "<p><a href="./">go to install</p>"
}


#
# main
#

html_header

if ! check_environment; then
	# environment check error
	echo 'environment error (suExec disabled etc.)'
else
	fetch_request
	validate_requests

	if [ -n "$REQUEST_extract" ] ; then
		extraction
	elif [ -n "$REQUEST_overwrite" ] ; then
		overwriting
	elif [ -n "$REQUEST_filename" ] ; then
		display_confirm
	else
		display_form
	fi
fi

html_footer
