#!/bin/bash
# This script is written by GIJOE <gij@peak.ne.jp>
# New BSD License


#
# initials
#

read POST_RAW

declare -r my_default_name='hd_updater.cgi'
declare -r download_base_url='http://downloads.sourceforge.net/hodajuku/'
declare -r archive_default_name=''
declare -r -a allowed_requests=('filename' 'chmodexecphp' 'pass' 'createbackup' 'extract' 'overwrite' 'owaction' 'owdirname' 'owtrustdirname' 'terminate')
declare -r rootname_of_archive='html'
declare -r trustname_of_archive='xoops_trust_path'
declare -r command_after_extraction=''
declare -r tmp4update='tmp4update'

REQUEST_filename=''
REQUEST_chmodexecphp=0
REQUEST_pass=''
REQUEST_createbackup=''
REQUEST_extract=''
REQUEST_overwrite=''
REQUEST_owaction=''
REQUEST_owdirname=''
REQUEST_owtrustdirname=''
REQUEST_terminate=''

MAINFILE_trust_path=''
MAINFILE_trust_dirname=''
MAINFILE_root_path=`pwd`
MAINFILE_root_dirname=`basename $MAINFILE_root_path`
MAINFILE_db_prefix=''
MAINFILE_salt=''


#
# functions
#

function check_environment() {
	# suExec enabled?
	if [ -O . ] ; then
		return 0;
	else
		return 1;
	fi
}

function parse_mainfile() {
	MAINFILE_trust_path=`grep -m1 XOOPS_TRUST_PATH mainfile.php|sed 's/define//g'|sed 's/XOOPS_TRUST_PATH//g'|sed 's/[^0-9a-zA-Z_/.-]//g'`
	MAINFILE_trust_dirname=`basename $MAINFILE_trust_path`
	MAINFILE_db_prefix=`grep -m1 XOOPS_DB_PREFIX mainfile.php|sed 's/define//g'|sed 's/XOOPS_DB_PREFIX//g'|sed 's/[^0-9a-zA-Z_]//g'`
	MAINFILE_salt=`grep -m1 XOOPS_SALT mainfile.php|sed s/define//g|sed s/[^0-9a-f]//g`
	MAINFILE_tmp_path="$MAINFILE_trust_path/$tmp4update"
}

function fetch_request() {
	local tmp k_v key value allowed_key

	QUERY_STRING=`echo "$QUERY_STRING"|sed 's/[^a-zA-Z0-9.&=%_-]//g'`
	POST_RAW=`echo "$POST_RAW"|sed 's/[^a-zA-Z0-9.&=%_-]//g'`

	tmp="$QUERY_STRING&$POST_RAW&"
	until [ -z "$tmp" ] ; do
		k_v=${tmp%%&*}
		tmp=${tmp#*&}
		key=${k_v%%=*}
		value=${k_v#*=}
		for allowed_key in ${allowed_requests[@]}; do
			if [ "$allowed_key" = "$key" ] ; then
				eval "REQUEST_$allowed_key=$value"
			fi
		done
	done
}

function validate_requests() {
	# you must eliminate HTML special characters '"&<>
	REQUEST_filename=`echo "$REQUEST_filename"|sed 's/[^a-zA-Z0-9._-]//g'`
	REQUEST_pass=`echo "$REQUEST_pass"|sed 's/[^a-f0-9]//g'`
	# you can use let as intval() of php
	# let REQUEST_trustindex=$REQUEST_trustindex
}

function html_header() {
	echo -e 'Content-Type: text/html'
	echo -e ''
	echo -e '<html>\n'
	echo -e '<head>\n'
	echo -e '<title>updater</title>\n'
	echo -e '<script type="text/javascript">\n'
	echo 'function turn_on_all_modules(){for(i=0;i<document.forms.length;i++){document.forms[i].style.display="block";}}'
	echo -e '</script>\n'
	echo -e '</head>\n'
	echo -e '<body>\n'
}

function html_footer() {
	echo -e '</body>\n</html>'
}

function display_auth_form() {
	echo '<form action="" method="post">'
	echo '<label>XOOPS_SALT in your mainfile.php:'
	echo '<input type="text" name="pass" value="" size="8" /></label><br />'
	echo '<input type="submit" value="login" />'
	echo '</form>'
}

function display_form() {
	echo '<form action="" method="post">'
	echo '<label>archive file name:'
	echo '<input type="text" name="filename" value="'$archive_default_name'" size="64" /></label><br />'
	echo 'root path: '$MAINFILE_root_path'<br />'
	echo 'trust path: '$MAINFILE_trust_path'<br />'
	echo 'need execute permissions for *.php?'
	echo '<label><input type="radio" name="chmodexecphp" value="0" checked="checked" />no</label>'
	echo '<label><input type="radio" name="chmodexecphp" value="1" />yes</label>'
	echo '<br />'
	echo '<input type="hidden" name="pass" value="'$REQUEST_pass'" />'
	echo '<input type="submit" name="createbackup" value="create a backup archive" />'
	echo '<input type="submit" name="extract" value="extract it" />'
	echo '</form>'
}

function create_backup() {
	local backup_root_filename backup_trust_filename

	backup_root_filename="$MAINFILE_trust_path/backup/rootfiles-"`date +%Y%m%d%H%M`.tar.gz
	backup_trust_filename="$MAINFILE_trust_path/backup/trustfiles-"`date +%Y%m%d%H%M`.tar.gz

	mkdir "$MAINFILE_trust_path/backup" 2>/dev/null
	tar czf $backup_root_filename * --exclude xoops_trust_path --exclude cache --exclude templates_c
	cd $MAINFILE_trust_path
	tar czf $backup_trust_filename * --exclude backup --exclude cache --exclude templates_c
	cd -

	echo '<p>You would better check the backup archive is made successfully</p>'
	echo '<ul><li>'$backup_root_filename'</li>'
	echo '<li>'$backup_trust_filename'</li></ul>'

	echo '<form action="" method="post">'
	echo '<input type="hidden" name="pass" value="'$REQUEST_pass'" />'
	echo '<input type="submit" value="back to the form" />'
	echo '</form>'
}

function extraction() {
	local from_remote

	rm -r $MAINFILE_tmp_path 2>/dev/null
	mkdir $MAINFILE_tmp_path
	cd $MAINFILE_tmp_path

	if [ -r "$MAINFILE_root_path/$REQUEST_filename" ] ; then
		# 1st: local file just under ROOT
		cp -p "$MAINFILE_root_path/$REQUEST_filename" $REQUEST_filename
		from_remote=false
	else
		# 2nd: try wget from URL
		wget -q $download_base_url$REQUEST_filename
		from_remote=true
	fi
	# 3rd: try curl from URL
	if [ ! -r $REQUEST_filename ] ; then
		curl -s -L $download_base_url$REQUEST_filename > $REQUEST_filename
	fi
	# last: raise an error
	if [ ! -r $REQUEST_filename ] ; then
		rm $REQUEST_filename 2>/dev/null
		echo '<p>cannot download $download_base_url$REQUEST_filename</p>'
		echo '<p>confirm the url</p>'
		exit 1
	fi

	# extract the archive
	if ! tar xpzf $REQUEST_filename ; then
		echo '<p>invalid archive type</p>'
		exit 1
	fi

	# check rootname
	if [ ! -x $rootname_of_archive ] ; then
		echo '<p>missing $rootname_of_archive folder in the archive </p>'
		exit 1
	fi

	# remove the archive
	rm $REQUEST_filename

	# chmod a+x *.php if requested
	if [ $REQUEST_chmodexecphp = '1' ] ; then
		find . -name '*.php' -exec chmod a+x {} \;
	fi

	# remove some fatal files just to make sure ...
	rm "$rootname_of_archive/mainfile.php" 2>/dev/null
	rm "$rootname_of_archive/.htaccess" 2>/dev/null
	rm -r "$rootname_of_archive/install" 2>/dev/null

	# chmod some files/directories if necessary
	eval "$command_after_extraction"

	cd -

	# message
	echo '<p>Both fetching and extraction are finished successfully</p>'
	display_copying_form
}

function display_copying_d3modules_form() {
	# form to correspod html/modules/dirname in the archive and site's dirname
	local mytrustdirname mytrustdirpath mydirname mydirpath

	if [ ! -x $MAINFILE_tmp_path/$trustname_of_archive/modules ] ; then
		return 1;
	fi

	echo '<dl><dt>D3 modules</dt><dd>'
	for mytrustdirpath in $MAINFILE_tmp_path/$trustname_of_archive/modules/* ; do # */
		mytrustdirname=`basename $mytrustdirpath`
		echo '<dl><dt>'$mytrustdirname'</dt><dd>'
		for mydirpath in $MAINFILE_root_path/modules/* ; do # */
			mydirname=`basename $mydirpath`
			if [ -r $mydirpath/mytrustdirname.php ] && grep $mytrustdirname $mydirpath/mytrustdirname.php 1>/dev/null ; then
				echo '<form action="" method="post">'
				echo '<input type="hidden" name="pass" value="'$REQUEST_pass'" />'
				echo '<input type="hidden" name="owaction" value="d3module" />'
				echo '<input type="hidden" name="owdirname" value="'$mydirname'" />'
				echo '<input type="hidden" name="owtrustdirname" value="'$mytrustdirname'" />'
				echo '<input type="submit" name="overwrite" value="-&gt; /modules/'$mydirname'/" />'
				echo '</form>'
			fi
		done
		# create a new copy of the d3module
		echo '<form action="" method="post">'
		echo '<input type="hidden" name="pass" value="'$REQUEST_pass'" />'
		echo '<input type="hidden" name="owaction" value="d3module" />'
		echo '<label>dirname:<input type="text" name="owdirname" value="" /></label>'
		echo '<input type="hidden" name="owtrustdirname" value="'$mytrustdirname'" />'
		echo '<input type="submit" name="overwrite" value="create a new copy" />'
		echo '</form>'
		echo '</dd></dl>'
	done
	echo '</dd></dl>'
}

function display_copying_modules_form() {
	local mydirname mydirpath form_style appendix_message
	echo '<dl><dt>single modules</dt><dd>'
	for mydirpath in $MAINFILE_tmp_path/$rootname_of_archive/modules/* ; do # */
		mydirname=`basename $mydirpath`
		# skip D3 modules
		if [ -r "modules/$mydirname/mytrustdirname.php" ] ; then continue ; fi
		# compare xoops_version.php
		if diff -b "modules/$mydirname/xoops_version.php" "$mydirpath/xoops_version.php" >/dev/null 2>/dev/null ; then
			form_style='display:none;'
			appendix_message='(up to date)'
		else
			form_style='display:block;'
			appendix_message=''
		fi
		echo '<dl id="modules"><dt>'$mydirname $appendix_message'</dt><dd>'
		echo '<form action="" method="post" style="'$form_style'">'
		echo '<input type="hidden" name="pass" value="'$REQUEST_pass'" />'
		echo '<input type="hidden" name="owaction" value="module" />'
		echo '<input type="hidden" name="owdirname" value="'$mydirname'" />'
		echo '<input type="submit" name="overwrite" value="-&gt; /modules/'$mydirname'/" />'
		echo '</form>'
		echo '</dd></dl>'
	done
	echo '<a href="javascript: turn_on_all_modules();">display all modules</a>'
	echo '</dd></dl>'
}

function display_copying_form() {
	display_copying_d3modules_form
	display_copying_modules_form

	echo '<dl><dt>system</dt><dd>'
	echo '<form action="" method="post">'
	echo '<input type="hidden" name="pass" value="'$REQUEST_pass'" />'
	echo '<input type="hidden" name="owaction" value="system" />'
	echo '<input type="submit" name="overwrite" value="overwrite all files except modules" onclick="return confirm(&#039;Are you OK?&#039;);" />'
	echo '<input type="submit" name="terminate" value="terminate transactions (clearing tmp folder)" />'
	echo '</form>'
	echo '</dd></dl>'
}

function overwriting() {
	cd "$MAINFILE_tmp_path"

	if [ "$REQUEST_owaction" = 'system' ] ; then
		# system
		for i in $rootname_of_archive/* ; do # */
			# skip modules/
			if [ "$i" = "$rootname_of_archive/modules" ] ; then continue ; fi
			cp -pR $i $MAINFILE_root_path
		done
		if [ -x $trustname_of_archive ] ; then
			for i in $trustname_of_archive/* ; do # */
				cp -pR $i $MAINFILE_trust_path
			done
		fi
	elif [ "$REQUEST_owaction" = 'module' ] && [ -n "$REQUEST_owdirname" ] ; then
		# single modules
		mkdir "$MAINFILE_root_path/modules/" 2>/dev/null
		cp -pR "$rootname_of_archive/modules/$REQUEST_owdirname" "$MAINFILE_root_path/modules/"
	elif [ "$REQUEST_owaction" = 'd3module' ] && [ -n "$REQUEST_owdirname" ] && [ -n "$REQUEST_owtrustdirname" ] ; then
		# D3 modules
		mkdir "$MAINFILE_root_path/modules/" 2>/dev/null
		mkdir "$MAINFILE_root_path/modules/$REQUEST_owdirname" 2>/dev/null
		cp -pR $rootname_of_archive/modules/$REQUEST_owtrustdirname/* "$MAINFILE_root_path/modules/$REQUEST_owdirname" # */
		cp -pR $rootname_of_archive/modules/$REQUEST_owtrustdirname/.[0-9a-zA-Z]* "$MAINFILE_root_path/modules/$REQUEST_owdirname"
	fi

	cd -

	# message
	echo '<p>System files are updated successfully.</p>'
	display_copying_form
}

function terminating() {
	# remove tmp4update
	rm -r "$MAINFILE_tmp_path" 2>/dev/null

	# message
	echo '<p>Updating transactions are finished.</p>'
	echo '<p><a href="./">return to site top</p>'
	echo '<p><a href="./'$my_default_name'">retry this updater</p>'
}


#
# main
#

html_header

if ! check_environment; then
	# environment check error
	echo 'environment error (suExec disabled etc.)'
else
	parse_mainfile
	fetch_request
	validate_requests

	if [ "$REQUEST_pass" != "$MAINFILE_salt" ] ; then
		display_auth_form
	elif [ -n "$REQUEST_createbackup" ] ; then
		create_backup
	elif [ -x "$MAINFILE_tmp_path/$rootname_of_archive" ] ; then
		if [ -n "$REQUEST_terminate" ] ; then
			terminating
		elif [ -n "$REQUEST_overwrite" ] ; then
			overwriting
		else
			display_copying_form
		fi
	elif [ -n "$REQUEST_filename" ] && [ -n "$REQUEST_extract" ] ; then
		extraction
	else
		display_form
	fi
fi

html_footer
